@@ -121,6 +121,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                                                                     connector=aiohttp.TCPConnector(ssl=False))
 
         self._api_version_getrec: int = 0
+        self._api_version_getftp: int = 0
         self._api_version_getpush: int = 0
         self._api_version_getalarm: int = 0
 
@@ -427,6 +428,11 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         else:
             body.append({"cmd": "GetPushV20", "action": 1, "param": {"channel": self._channel}})
 
+        if self._api_version_getftp == 0:
+            body.append({"cmd": "GetFtp", "action": 1, "param": {"channel": self._channel}})
+        else:
+            body.append({"cmd": "GetFtpV20", "action": 1, "param": {"channel": self._channel}})
+
         if self._api_version_getrec == 0:
             body.append({"cmd": "GetRec", "action": 1, "param": {"channel": self._channel}})
         else:
@@ -483,7 +489,6 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         try:
             json_data = json.loads(response)
             self.map_json_response(json_data)
-            return True
         except (TypeError, json.JSONDecodeError):
             _LOGGER.debug(
                 "Host %s: Error translating Reolink settings response", self._host
@@ -491,6 +496,51 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
             self.clear_token()
             return False
 
+        # checking API versions (because Reolink dev quality sucks big time we cannot fully trust GetAbility)
+        body = []
+        if self._api_version_getpush == 1:
+            body.append({"cmd": "GetPushV20", "action": 1, "param": {"channel": self._channel}})
+        if self._api_version_getftp == 1:
+            body.append({"cmd": "GetFtpV20", "action": 1, "param": {"channel": self._channel}})
+        if self._api_version_getrec == 1:
+            body.append({"cmd": "GetRecV20", "action": 1, "param": {"channel": self._channel}})
+        if self._api_version_getalarm == 1:
+            body.append({"cmd": "GetAudioAlarmV20", "action": 1, "param": {"channel": self._channel}})
+
+        response = await self.send(body)
+        try:
+            json_data = json.loads(response)
+        except (TypeError, json.JSONDecodeError):
+            _LOGGER.debug(
+                "Host %s: Error translating Reolink settings response", self._host
+            )
+            self.clear_token()
+            return False
+
+        def check_command_exists(cmd: str):
+            for x in json_data:
+                if x["cmd"] == cmd:
+                    return True
+            return False
+
+        if self._api_version_getpush == 1:
+            if not check_command_exists("GetPushV20"):
+                self._api_version_getpush = 0
+
+        if self._api_version_getftp == 1:
+            if not check_command_exists("GetFtpV20"):
+                self._api_version_getftp = 0
+
+        if self._api_version_getrec == 1:
+            if not check_command_exists("GetRecV20"):
+                self._api_version_getrec = 0
+
+        if self._api_version_getalarm == 1:
+            if not check_command_exists("GetAudioAlarmV20"):
+                self._api_version_getalarm = 0
+
+        return True
+
     async def get_motion_state(self):
         """Fetch the motion state."""
         body = [{"cmd": "GetMdState", "action": 0, "param": {"channel": self._channel}}]
@@ -572,24 +622,24 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
     def get_rtmp_stream_source(self) -> str:
         if self._rtmp_auth_method == DEFAULT_RTMP_AUTH_METHOD:
-            return f"rtmp://{self._host}:{self._rtmp_port}/bcs/channel{self._channel}_{self._stream}.bcs?channel=\
-                    {self._channel}&stream=0&user={self._username}&password={self._password}"
+            return f"rtmp://{self._host}:{self._rtmp_port}/bcs/channel{self._channel}_{self._stream}.bcs?channel=" \
+                   f"{self._channel}&stream=0&user={self._username}&password={self._password}"
 
-        return f"rtmp://{self._host}:{self._rtmp_port}/bcs/channel{self._channel}_{self._stream}.bcs?channel=\
-                    {self._channel}&stream=0&token={self._token}"
+        return f"rtmp://{self._host}:{self._rtmp_port}/bcs/channel{self._channel}_{self._stream}.bcs?channel=" \
+               f"{self._channel}&stream=0&token={self._token}"
 
     def get_rtsp_stream_source(self) -> str:
         password = parse.quote(self._password)
         channel = "{:02d}".format(self._channel + 1)
-        return f"rtsp://{self._username}:{password}@{self._host}:{self._rtsp_port}/\
-                {self._stream_format}Preview_{channel}_{self._stream}"
+        return f"rtsp://{self._username}:{password}@{self._host}:{self._rtsp_port}/" \
+               f"{self._stream_format}Preview_{channel}_{self._stream}"
 
     async def get_stream_source(self):
         """Return the stream source url."""
         if not await self.login():
             return
 
-        if self.protocol == DEFAULT_PROTOCOL:
+        if self.protocol == "rtmp":
             stream_source = self.get_rtmp_stream_source()
         else:
             stream_source = self.get_rtsp_stream_source()
@@ -614,11 +664,6 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
     def map_json_response(self, json_data):  # pylint: disable=too-many-branches
         """Map the JSON objects to internal objects and store for later use."""
 
-        push_data = None
-        pushv20_data = None
-        rec_data = None
-        recv20_data = None
-
         for data in json_data:
             try:
                 if data["code"] == 1:  # -->Error, like "ability error"
@@ -665,11 +710,17 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                     self._ftp_settings = data
                     self._ftp_state = data["value"]["Ftp"]["schedule"]["enable"] == 1
 
+                elif data["cmd"] == "GetFtpV20":
+                    self._ftp_settings = data
+                    self._ftp_state = data["value"]["Ftp"]["enable"] == 1
+
                 elif data["cmd"] == "GetPush":
-                    push_data = data
+                    self._push_settings = data
+                    self._push_state = data["value"]["Push"]["schedule"]["enable"] == 1
 
                 elif data["cmd"] == "GetPushV20":
-                    pushv20_data = data
+                    self._push_settings = data
+                    self._push_state = data["value"]["Push"]["enable"] == 1
 
                 elif data["cmd"] == "GetEnc":
                     self._enc_settings = data
@@ -714,11 +765,14 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                     self._motion_detection_state = data["value"]["Alarm"]["enable"] == 1
                     self._sensitivity_presets = data["value"]["Alarm"]["sens"]
 
-                elif data["cmd"] == "GetAudioAlarm" or data["cmd"] == "GetAudioAlarmV20":
+                elif data["cmd"] == "GetAudioAlarm":
+                    self._audio_alarm_settings = data
+                    self._audio_alarm_state = data["value"]["Audio"]["schedule"]["enable"]
+
+                elif data["cmd"] == "GetAudioAlarmV20":
                     self._audio_alarm_settings = data
                     self._audio_alarm_state = data["value"]["Audio"]["enable"]
 
-
                 elif data["cmd"] == "GetAbility":
                     for ability in data["value"]["Ability"]["abilityChn"]:
                         self._ptz_support = ability["ptzCtrl"]["permit"] != 0
@@ -730,8 +784,10 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                             self._api_version_getpush = details['ver']
                         elif ability == 'supportRecordEnable':
                             self._api_version_getrec = details['ver']
-                        elif  ability == 'scheduleVersion':
+                        elif ability == 'scheduleVersion':
                             self._api_version_getalarm = details['ver']
+                        elif ability == 'supportFtpEnable':
+                            self._api_version_getftp = details['ver']
 
                 elif data["cmd"] == "GetNtp":
                     self._ntp_settings = data
@@ -743,13 +799,6 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                 _LOGGER.error(traceback.format_exc())
                 continue
 
-        if pushv20_data is not None:
-            self._push_settings = pushv20_data
-            self._push_state = pushv20_data["value"]["Push"]["enable"] == 1
-        elif push_data is not None:
-            self._push_settings = push_data
-            self._push_state = push_data["value"]["Push"]["schedule"]["enable"] == 1
-
     async def login(self):
         """Login and store the session ."""
         if self.session_active:
@@ -998,22 +1047,6 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         return await self.send_setting(body)
 
-    async def set_ftp(self, enable):
-        """Set the FTP parameter."""
-        if not self._ftp_settings:
-            _LOGGER.error("Actual FTP settings not available")
-            return False
-
-        if enable:
-            new_value = 1
-        else:
-            new_value = 0
-
-        body = [{"cmd": "SetFtp", "action": 0, "param": self._ftp_settings["value"]}]
-        body[0]["param"]["Ftp"]["schedule"]["enable"] = new_value
-
-        return await self.send_setting(body)
-
     async def set_push(self, enable: bool):
         """Set the PUSH (notifications) parameter."""
         if not self._push_settings:
@@ -1034,6 +1067,26 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         return await self.send_setting(body)
 
+    async def set_ftp(self, enable: bool):
+        """Set the FTP (notifications) parameter."""
+        if not self._ftp_settings:
+            _LOGGER.error("Actual FTP settings not available")
+            return False
+
+        if enable:
+            new_value = 1
+        else:
+            new_value = 0
+
+        if self._api_version_getftp == 0:
+            body = [{"cmd": "SetFtp", "action": 0, "param": self._ftp_settings["value"]}]
+            body[0]["param"]["Ftp"]["schedule"]["enable"] = new_value
+        else:
+            body = [{"cmd": "SetFtpV20", "action": 0, "param": self._ftp_settings["value"]}]
+            body[0]["param"]["Ftp"]["enable"] = new_value
+
+        return await self.send_setting(body)
+
     async def set_audio(self, enable):
         """Set the audio parameter."""
         if not self._enc_settings:
@@ -1179,9 +1232,15 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
             return await self.get_states()
 
     async def set_spotlight(self, enable):
-        # simply calls set_whiteled with brightness 100, mode 1
+        # simply calls set_whiteled with brightness 100, mode 3
+        # after setting lightning schedule to on all the time 0000 to 2359
 
-        return await self.set_whiteled(enable, 100, 1)
+        if enable:
+            if not await self.set_spotlight_lighting_schedule(23, 59, 0, 0):
+                return False
+            return await self.set_whiteled(enable, 100, 3)
+        else:
+            return await self.set_whiteled(enable, 100, 1)
 
     async def set_audio_alarm(self,enable, *args):
         # fairly basic only either turns it off or on
@@ -1200,12 +1259,12 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         if self._api_version_getalarm == 0:
             body = [
-                    {'cmd': 'SetAudioAlarm',
-                        'param': { "Audio":
-                            {"schedule": {"enable": on_off}}
-                                }
-                    }
-                    ]
+                {'cmd': 'SetAudioAlarm',
+                 'param': {"Audio":
+                               {"schedule": {"enable": on_off}}
+                           }
+                 }
+            ]
         else:
             body = [
                 {'cmd': 'SetAudioAlarmV20',
@@ -1222,8 +1281,35 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
             return self.get_settings()
 
     async def set_siren(self,enable):
-        # just calls set_audio_alarm with a dummy *args argument
-        return await self.set_audio_alarm(enable,  False)
+        # Uses API AudioAlarmPlay with manual switch
+        # uncertain if there may be a glitch - dont know if there is API I have yet to find
+        # which sets AudioLevel
+        if enable:
+            man_switch = 1
+        else:
+            man_switch = 0
+
+        # this is overkill but to get state set right necessary to call set_audio_alarm
+
+        if not await self.set_audio_alarm(enable):
+            return False
+
+        body = [
+            {'cmd': 'AudioAlarmPlay',
+              'action': 0,
+             'param': {
+                 "alarm_mode": 'manul',
+                 'manual_switch': man_switch,
+                 'times': 2,
+                 "channel": 0
+                       }
+             }
+        ]
+
+        if not await self.send_setting(body):
+            return False
+        else:
+            return self.get_settings()
 
     async def set_daynight(self, value):
         """Set the daynight parameter."""
